#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Code to make WebP animations of specific regions of TROPOMI NO2 Tropospheric Column

V03--Add the cropping and the count plume function
V02--Generates Webp as the output
V01--Working only for the Canary Island Region and only makes GIF


Authors: 
Santiago Ramirez Agudelo  BSC
Andrés Yarce Botero  BSC 
"""
import os
from datetime import datetime, timedelta
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains
from PIL import Image
import time

import cv2
import numpy as np
import matplotlib.pyplot as plt

# Function to generate URLs for a given date range
def generate_urls(start_date, end_date):
    base_url = "https://mpc-l2.tropomi.eu/map_v2.html?map.id=tropomi/"
    date_format = "%Y/%m/%d"
    urls = []
    current_date = start_date
    while current_date <= end_date:
        date_str = current_date.strftime(date_format)
        url = f"{base_url}{date_str}/nitrogendioxide_tropospheric_column"
        urls.append(url)
        current_date += timedelta(days=1)
    return urls

# Capture map images from URL
def capture_map_image(url, save_path):
    # Set up Selenium WebDriver
    options = webdriver.ChromeOptions()
    options.add_argument('--headless')
    options.add_argument('--window-size=1920x1080')  # Set window size to ensure full map is visible
    service = Service('/home/ayarce/.wdm/drivers/chromedriver/linux64/127.0.6533.72/chromedriver-linux64/chromedriver')
    driver = webdriver.Chrome(service=service, options=options)
    try:
        # Load the webpage
        driver.get(url)
        # Wait for the opacity change to take effect
        driver.implicitly_wait(2)
        # Click on the map to ensure it has focus
        map_element = driver.find_element(By.ID, 'map')
        actions = ActionChains(driver)
        actions.move_to_element(map_element).click().perform()
        # Press UP and LEFT arrow keys with a delay of 0.5 seconds
        time.sleep(0.1)
        actions.send_keys(Keys.ARROW_UP).perform()
        time.sleep(0.1)
        actions.send_keys(Keys.ARROW_LEFT).perform()
        # Select the zoom-in control and click it 5 times with a delay
        zoom_in_element = driver.find_element(By.CLASS_NAME, 'leaflet-control-zoom-in')
        for _ in range(5):
            time.sleep(0.1)
            zoom_in_element.click()
        # Press DOWN arrow key 10 times with a delay of 0.1 seconds
        for _ in range(10):
            time.sleep(0.1)
            actions.send_keys(Keys.ARROW_DOWN).perform()
        # Change the opacity of the first leaflet-layer element
        script_opacity = """
        var element = document.getElementsByClassName('leaflet-layer')[1];
        element.style.opacity = '1';
        """
        driver.execute_script(script_opacity)
        # Capture the screenshot of the map
        driver.save_screenshot(save_path)
    finally:
        # Clean up
        driver.quit()
        
        
# Function to create contour over an image and count plumes
def cropping(image_folder):     
    for file_name in sorted(os.listdir(image_folder)):
        if file_name.endswith('.png'):
            file_path = os.path.join(image_folder, file_name)
            img = Image.open(file_path)  # Open the image once
           
            # Get image dimensions
            width, height = img.size

            # Define the height of the lower band you want to cut
            band_height = 60  # Example: 50 pixels
            band_width = 60  # Example: 50 pixels

            # Calculate the cropping box (left, upper, right, lower)
            crop_box = (0, 0, width - band_width, height - band_height)

            # Crop the image
            cropped_image = img.crop(crop_box)
            
            # Define the path to save the cropped image
            cropped_image_path = os.path.join('/home/ayarce/Plume_Project_clean/ddeq/Tools/maps/croppeds', file_name)

            # Save the cropped image
            
            cropped_image.save(cropped_image_path)

            print(f"Cropped image saved as {cropped_image_path}")





# Function to create WebP from images with consistent color palette
def create_webp(image_folder, webp_path):
    images = []
    
    for file_name in sorted(os.listdir(image_folder)):
        if file_name.endswith('.png'):
            file_path = os.path.join(image_folder, file_name)
            img = Image.open(file_path)
            
            # Convert to RGB to ensure consistent color mode
            img = img.convert('RGB')
            
            images.append(img)
    
    # Save as WebP animation with fixed duration and infinite loop
    images[0].save(webp_path, save_all=True, append_images=images[1:], duration=1000, loop=0, format='WEBP')





# Function to process a single image and detect high concentration areas
def detect_plumes(image_path, output_folder, image_index):
    # Load the image
    image = cv2.imread(image_path)

    # Convert the image to HSV (Hue, Saturation, Value) color space to better isolate the red color
    hsv_image = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)

    # Define the range for the red color in the HSV color space
    lower_red = np.array([0, 100, 100])
    upper_red = np.array([10, 255, 255])

    # Create a mask for red areas
    red_mask = cv2.inRange(hsv_image, lower_red, upper_red)

    # Find contours in the red mask
    red_contours, _ = cv2.findContours(red_mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

    # Draw the contours on the original image and label them with numbers
    labeled_red_image = image.copy()

    # Loop over the contours and draw each with a number
    for i, contour in enumerate(red_contours):
        # Draw the contour
        cv2.drawContours(labeled_red_image, [contour], -1, (0, 255, 0), 2)
        
        # Find the center of the contour and put the label
        M = cv2.moments(contour)
        if M["m00"] != 0:
            cX = int(M["m10"] / M["m00"])
            cY = int(M["m01"] / M["m00"])
            cv2.putText(labeled_red_image, str(i + 1), (cX - 10, cY + 10),
                        cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 0), 2)

    # Save the labeled image to the output folder with a new filename
    output_filename = f"count_plumes_{image_index:02d}.png"
    output_path = os.path.join(output_folder, output_filename)
    cv2.imwrite(output_path, labeled_red_image)




# Define date range and output directory
start_date = datetime(2024, 1, 1)
end_date = datetime(2024, 1, 2)
output_dir = "maps"

# Create output directory if it doesn't exist
os.makedirs(output_dir, exist_ok=True)

# Generate URLs
urls = generate_urls(start_date, end_date)

# Capture and save map images for each URL
for url in urls:
    date_str = url.split("/")[-2]
    save_path = os.path.join(output_dir, f"map_{date_str}.png")
    capture_map_image(url, save_path)
    cropping(output_dir)


# Create WebP animation with consistent color palette
webp_path = os.path.join(output_dir, "map_sequence.webp")
create_webp(output_dir, webp_path)


# Directory containing the images
image_cropped = "/home/ayarce/Plume_Project_clean/ddeq/Tools/maps/croppeds"


# Create the output directory inside the image folder
output_folder = os.path.join(image_cropped, "count_plumes")
os.makedirs(output_folder, exist_ok=True)

# Process each image in the folder
index=0
for index, filename in enumerate(sorted(os.listdir(image_cropped))):
    if filename.endswith((".png", ".jpg", ".jpeg")):  # Add more extensions if needed
        image_path = os.path.join(image_cropped, filename)
        detect_plumes(image_path, output_folder, index + 1)

print(f"Processed images have been saved in: {output_folder}")